/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HW3_FORMREADER_H_
#define _HW3_FORMREADER_H_

#include <string>

namespace hw3 {

// This class is used to handle form submissions from the
// main index.html page.  It tests the URI to see whether
// the "list of results" or "wordcloud" version of the results
// has been requested, issues a twitter search, and prepares
// the HTML of the results.
class FormReader {
 public:
  // Stash away the URI, but don't do any processing.
  explicit FormReader(std::string uri) : uri_(uri) { }
  virtual ~FormReader(void) { }

  // Process the query, produce HTML results and return them through
  // the "str" output parameter.  Returns "true" on success and
  // "false" if the form arguments are not recognized.
  //
  // If the URI is of the form:
  //
  //    /post?twittersearch=%22test+query%22
  //
  // then it should issue a twitter search for "test query" and
  // generate an HTML page that lists the search results.
  //
  // If the URI is of the form:
  //
  //    /post?twittercloud=test+query
  //
  // then it should issue a twitter search for "test query" and
  // generate an HTML page with a word cloud with the results, using
  // your code from HW2.
  bool ProcessForm(std::string *str);

 private:
  std::string uri_;
};

}  // namespace hw3

#endif  // _HW3_FORMREADER_H_
