/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <cstdlib>
#include <cstdio>
#include <iostream>

#include "./ServerSocket.h"
#include "./HttpServer.h"

using std::cerr;
using std::cout;
using std::endl;

// Print out program usage, and exit() with EXIT_FAILURE.
void Usage(char *progname);

// Parses the command-line arguments, invokes Usage() on failure.
// "port" is a return parameter to the port number to listen on,
// and "path" is a return parameter to the directory containing
// our static files.
void GetPortAndPath(int argc,
                    char **argv,
                    uint16_t *port,
                    std::string *path);

int main(int argc, char **argv) {
  // Print out welcome message.
  cout << "Welcome to http333d, the UW cse333 web server!" << endl;
  cout << "  Copyright 2012 Steven Gribble" << endl;
  cout << "  http://www.cs.washington.edu/homes/gribble" << endl;
  cout << endl;
  cout << "initializing:" << endl;
  cout << "  parsing port number and static files directory..." << endl;

  // Get the port number and list of index files.
  uint16_t portnum;
  std::string staticdir;
  GetPortAndPath(argc, argv, &portnum, &staticdir);
  cout << "    port: " << portnum << endl;
  cout << "    path: " << staticdir << endl;

  // Run the server.
  hw3::HttpServer hs(portnum, staticdir);
  if (!hs.Run()) {
    cerr << "  server failed to run!?" << endl;
  }

  cout << "server completed!  Exiting." << endl;
  return EXIT_SUCCESS;
}


void Usage(char *progname) {
  cerr << "Usage: " << progname << " port staticfiles_directory";
  cerr << endl;
  exit(EXIT_FAILURE);
}

void GetPortAndPath(int argc,
                    char **argv,
                    uint16_t *port,
                    std::string *path) {
  // Do we have the right number of command line arguments?
  if (argc != 3) {
    cerr << endl;
    Usage(argv[0]);
  }

  // Try to get the port number.
  if (sscanf(argv[1], "%hu", port) != 1) {
    cerr << endl << argv[1] << " isn't a valid port number." << endl;
    Usage(argv[0]);
  }

  // Test to see if "path" is a readable directory.
  struct stat fs;
  if ((stat(argv[2], &fs) == -1) ||
      (!S_ISDIR(fs.st_mode))) {
    cerr << endl << argv[2] << " isn't a directory." << endl;
    Usage(argv[0]);
  }
  DIR *d = opendir(argv[2]);
  if (d == NULL) {
    cerr << endl << argv[2] << " isn't a readable directory." << endl;
    Usage(argv[0]);
  }
  closedir(d);
  *path = argv[2];
}

