/*
 * Copyright 2012 Steven Gribble
 *
 *  This file is part of the UW CSE333 project sequence (333proj12).
 *
 *  333proj12 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  333proj12 is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj12.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HW2_TWITTERCLIENT_H_
#define _HW2_TWITTERCLIENT_H_

#include <boost/shared_ptr.hpp>
#include <stdint.h>
#include <string>
#include <vector>

#include "./HTTPClient.h"
#include "./JSONParser.h"
#include "./Utils.h"

class TwitterSearch {
 public:
  explicit TwitterSearch(std::string searchterm);
  ~TwitterSearch() { }

  // A class defining the search result fields.  A field is blank ("")
  // if it was not provided during the search or if it has a null value.
  // String values are converted to (non-encoded) strings.
  class Result {
  public:
    std::string ToString();        // return a fully-formed string rep.

    std::string created_at;        // when the was tweet created
    std::string from_user;         // user tag of sender
    std::string from_user_id_str;  // userid of sender
    std::string from_user_name;    // name of the sender
    std::string geo;               // geo tag of user
    std::string location;          // location tag of user
    std::string id_str;            // tweet ID
    std::string source;            // tweet source
    std::string text;              // tweet text
    std::string to_user;           // user tag of the receiver
    std::string to_user_id_str;    // userid of the receiver
    std::string to_user_name;      // user name of the receiver
  };

  // Did the search succeed?
  bool success_;

  // The vector of search results.
  std::vector<Result> result_vec_;

  // Raw access to the underlying JSONValue pointer.
  boost::shared_ptr<JSONValue> result_value_;

 private:

  DISALLOW_COPY_AND_ASSIGN(TwitterSearch);
};

#endif  // _HW2_TWITTERCLIENT_H_
