/*
 * Copyright 2012 Steven Gribble
 *
 *  This file is part of the UW CSE333 project sequence (333proj12).
 *
 *  333proj12 is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation, either version 3 of the License,
 *  or (at your option) any later version.
 *
 *  333proj12 is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj12.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _HW2_UTILS_H_
#define _HW2_UTILS_H_

#include <string>

// A macro to disallow the copy constructor and operator= functions.
// This should be used in the private: declarations for a class.
#define DISALLOW_COPY_AND_ASSIGN(TypeName) \
  TypeName(const TypeName&);               \
  void operator=(const TypeName&)

// Takes an unescaped, UTF-8 encoded, NULL-terminated string,
// and converts it into a JSON-escaped string.  In other words,
// quotes are replaced with \", and so on.
//
// Arguments:
//
// - unescaped:  the string to escape
//
// Returns:
//
// - an escaped string.
std::string JSONEscapeString(unsigned char *unescaped);

// Takes a JSON escaped, NULL-terminated string, and converts it into
// an unescaped UTF-8 encoded std::string.  In other words, sequences like
// \" are replaced with ", and so on.
//
// Arguments:
//
// - escaped:  the string to unescape
//
// Returns:
//
// - an unescaped string
std::string JSONUnescapeString(unsigned char *escaped);

// Convert a UTF-8 encoded sequence into a NULL-terminated \uWXYZ-style
// UTF-16 encoding, as required by the JSON spec.  Caller must provide both
// the input and output buffers.  Returns the number of bytes of UTF-8
// data that was consumed.
int UTF8ToSlashUEncoded(unsigned char *bytes, char buf[7]);

// Convert a NULL-terminated \uWXYZ-style UTF-16 encoding to a
// NULL-terminated UTF-8 encoding sequence.  Caller must provide
// both the input and output buffers.
void SlashUEncodedToUTF8(char *bytes, unsigned char buf[4]);

// Perform URI encoding on a string.  URL encoding, also known as
// percent-encoding, is a mechanism for converting an arbitrary
// string into something that is safe to include as a component of
// a URL, such as the URLs we generate to search twitter.  See:
//
//    http://en.wikipedia.org/wiki/Percent-encoding
//
// for details about how URIEncoding works.
std::string URIEncode(const std::string &src);

// This function performs HTML escaping in place.  It scans a string
// for dangerous HTML tokens (such as "<") and replaces them with the
// escaped HTML equivalent (such as "&lt;").  This helps to prevent
// XSS attacks in any HTML that we generate.
std::string EscapeHTMLInPlace(const std::string &from);

#endif  // _HW2_UTILS_H_
